/*
    Copyright (c) 2010, 2011 aizo ag, Zurich, Switzerland

    Author: Sergey 'Jin' Bostandzhyan <jin@dev.digitalstrom.org>

    This file is part of dS Assistant.

    dS-Assistant is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    dS-Assistant is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.

    NOTE: all APIs, especially the JSON API, offered by this application
          are subject to change without further notice.
*/

#ifndef __HWDESC_H__
#define __HWDESC_H__

#include <boost/shared_ptr.hpp>
#include <digitalSTROM/ds.h>
#include <inttypes.h>
#include <string>
#include <vector>

/// \file hwdesc.h
/// \brief Hardware description classes.

namespace dSA
{

typedef enum
{
  Unknown,
  dSMeter,
  dSDevice,
} device_type;
/// base class for all dS hardware
class HWBase
{
public:
    HWBase(dsid_t dsid);
    HWBase();
    virtual ~HWBase();

    const dsid_t getDSID() const;
    void setDSID(dsid_t dsid);
    const device_type getDeviceType() const;

protected:
    dsid_t m_dsid;
    device_type m_device_type;
};

/// dS-Meter
class dSM : public HWBase
{
public:
    dSM(dsid_t dsid);

    void setHWVersion(uint32_t hw_version);
    uint32_t getHWVersion() const;
    void setARMSwVersion(uint32_t arm_sw_version);
    uint32_t getARMSwVersion() const;
    void setDSPSwVersion(uint32_t dsp_sw_version);
    uint32_t getDSPSwVersion() const;
    void setAPIVersion(uint16_t api_version);
    uint16_t getAPIVersion() const;
    void setName(std::string name);
    std::string getName() const;
    void setPresent(uint8_t present);
    uint8_t getPresent() const;
    void setApartmentId(uint8_t id);
    uint8_t getApartmentId() const;
    void setConfigurationId(uint64_t id);
    uint64_t getConfigurationId() const;

    void setDeviceCount(uint16_t);
    uint16_t getDeviceCount() const;
    void setActiveDeviceCount(uint16_t);
    uint16_t getActiveDeviceCount() const;
    void setInactiveDeviceCount(uint16_t);
    uint16_t getInactiveDeviceCount() const;

    void setZoneId(uint8_t index, uint16_t id);
    std::vector<uint16_t> getZoneIds() const;

protected:
    uint32_t m_hw_version;
    uint32_t m_arm_sw_version;
    uint32_t m_dsp_sw_version;
    uint16_t m_api_version;
    std::string m_name;
    std::vector<uint16_t> m_zone_ids;
    uint8_t m_present;
    uint8_t m_apartment_id;
    uint64_t m_config_hash;

    uint16_t m_count_devices;
    uint16_t m_count_active_devices;
    uint16_t m_count_inactive_devices;
};

/// "Klemme"
class Device : public HWBase
{
public:
    Device();
    Device(dsid_t dsid, dsid_t dsm_dsid);

    const dsid_t getDSMdSID() const;
    void setDSMdSID(dsid_t dsid);
    void setId(uint16_t device_id);
    uint16_t getId() const;
    void setFunctionId(uint16_t function_id);
    uint16_t getFunctionId() const;
    void setProductId(uint16_t product_id);
    uint16_t getProductId() const;
    void setVersion(uint16_t version);
    uint16_t getVersion() const;
    void setOutputMode(uint8_t output_mode);
    uint8_t getOutputMode() const;
    void setZoneId(uint16_t zone_id);
    uint16_t getZoneId() const;
    void setState(uint8_t state);
    uint8_t getState() const;
    void setLocked(uint8_t locked);
    uint8_t getLocked() const;
    void setName(const std::string name);
    std::string getName() const;
    void setSerialNumber(uint32_t number);
    uint32_t getSerialNumber() const;
    void setLTMode(uint8_t mode);
    uint8_t getLTMode() const;
    void getButtonInfo(uint8_t *input_count, uint8_t *input_index) const;
    std::string getHardwareInfoString() const;

protected:
    dsid_t m_dsm_dsid;
    uint8_t m_zone_index;

    // DeviceInfo
    uint16_t m_device_id;
    uint16_t m_function_id;
    uint16_t m_product_id;
    uint16_t m_version;
    uint16_t m_zone_id;
    uint8_t m_state;
    uint8_t m_locked;
    std::string m_name;
    uint8_t m_output_mode;
    uint32_t m_serial_number;
    uint8_t m_lt_mode;
    static const uint8_t m_max_colors;
    static const uint8_t m_max_devtypes;
    static const char * const m_colors[];
    static const char * const m_devtypes[];
};

};

#endif//__HWDESC_H__
