/*
    Copyright (c) 2012 aizo ag, Zurich, Switzerland

    Author: Sergey 'Jin' Bostandzhyan <jin@dev.digitalstrom.org>

    This file is part of dS Assistant.

    dS-Assistant is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    dS-Assistant is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.

    NOTE: all APIs, especially the JSON API, offered by this application
          are subject to change without further notice.
*/

#ifdef HAVE_CONFIG_H
#include "autoconfig.h"
#endif

#include "json_util.h"
#include "hwdesc.h"
#include "exceptions.h"
#include "util.h"

namespace dSA
{

json_object *jsonutil::device2json(boost::shared_ptr<const Device> device)
{
    uint8_t input_count;
    uint8_t input_index;

    if (device == NULL)
    {
        throw _Exception("invalid device");
    }

    json_object *jdevice = json_object_new_object();

    json_object_object_add(jdevice, "dsid",
        json_object_new_string(util::str(device->getDSID()).c_str()));

    json_object_object_add(jdevice, "dsmeter",
        json_object_new_string(util::str(device->getDSMdSID()).c_str()));

    json_object_object_add(jdevice, "functionId",
            json_object_new_int(device->getFunctionId()));

    json_object_object_add(jdevice, "productId",
            json_object_new_int(device->getProductId()));

    json_object_object_add(jdevice, "version",
            json_object_new_int(device->getVersion()));

    json_object_object_add(jdevice, "outputMode",
            json_object_new_int(device->getOutputMode()));

    json_object_object_add(jdevice, "zoneId",
            json_object_new_int(device->getZoneId()));

    json_object_object_add(jdevice, "serialNumber",
            json_object_new_int(device->getSerialNumber()));

    json_object_object_add(jdevice, "hardwareInfo",
            json_object_new_string(device->getHardwareInfoString().c_str()));

    json_object_object_add(jdevice, "present",
            json_object_new_boolean((bool)device->getState()));

    json_object_object_add(jdevice, "buttonInputMode",
            json_object_new_int((int)device->getLTMode()));

    device->getButtonInfo(&input_count, &input_index);

    json_object_object_add(jdevice, "buttonInputCount",
            json_object_new_int(input_count));

    json_object_object_add(jdevice, "buttonInputIndex",
            json_object_new_int(input_index));

    return jdevice;
}

} // namespace
