/*
    Copyright (c) 2011 aizo ag, Zurich, Switzerland

    Author: Sergey 'Jin' Bostandzhyan <jin@dev.digitalstrom.org>

    This file is part of dS Assistant.

    dS-Assistant is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    dS-Assistant is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.

    NOTE: all APIs, especially the JSON API, offered by this application
          are subject to change without further notice.

    The timer class is heavily inspired by timer code in the MediaTomb project:
    http://mediatomb.cc/
*/

#ifndef __TIMER_H__
#define __TIMER_H__

#include <pthread.h>
#include <sys/time.h>
#include <boost/shared_ptr.hpp>
#include <list>

namespace dSA
{

class TimerSubscriber
{
public:
    TimerSubscriber(unsigned int notifyInterval, bool once = true);
    virtual void notify() = 0;
    struct timespec *getNextNotify();
    unsigned int getNotifyInterval();
    void notified();
    bool once();
    virtual ~TimerSubscriber();

protected:
    TimerSubscriber();

private:
    struct timespec m_nextNotify;
    unsigned int m_notifyInterval;
    bool m_once;
};

class Timer
{
public:
    Timer();
    ~Timer();
    static Timer *createInstance();
    static Timer *getInstance();

    void addSubscriber(boost::shared_ptr<TimerSubscriber> ts);
    void removeSubscriber(boost::shared_ptr<TimerSubscriber> ts);
    void wait();
    void signal();

private:
    static Timer *m_instance;
    pthread_mutex_t m_subscribers_mutex;
    std::list<boost::shared_ptr<TimerSubscriber> > m_subscribers;
    pthread_cond_t m_notify_cond;

    void lock();
    void unlock();
    void notifySubscribers();
};

} // namespace

#endif//__TIMER_H__
